/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/mozilla-central-linux64-xulrunner/build/dom/indexedDB/nsIIDBEvent.idl
 */

#ifndef __gen_nsIIDBEvent_h__
#define __gen_nsIIDBEvent_h__


#ifndef __gen_nsIDOMEvent_h__
#include "nsIDOMEvent.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIIDBEvent */
#define NS_IIDBEVENT_IID_STR "d5623e5f-e19d-4234-8259-343a5a21c83a"

#define NS_IIDBEVENT_IID \
  {0xd5623e5f, 0xe19d, 0x4234, \
    { 0x82, 0x59, 0x34, 0x3a, 0x5a, 0x21, 0xc8, 0x3a }}

class NS_NO_VTABLE NS_SCRIPTABLE nsIIDBEvent : public nsIDOMEvent {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIDBEVENT_IID)

  /* readonly attribute nsISupports source; */
  NS_SCRIPTABLE NS_IMETHOD GetSource(nsISupports **aSource) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIDBEvent, NS_IIDBEVENT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIIDBEVENT \
  NS_SCRIPTABLE NS_IMETHOD GetSource(nsISupports **aSource); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIIDBEVENT(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetSource(nsISupports **aSource) { return _to GetSource(aSource); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIIDBEVENT(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetSource(nsISupports **aSource) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSource(aSource); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIDBEvent : public nsIIDBEvent
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIIDBEVENT

  nsIDBEvent();

private:
  ~nsIDBEvent();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsIDBEvent, nsIIDBEvent)

nsIDBEvent::nsIDBEvent()
{
  /* member initializers and constructor code */
}

nsIDBEvent::~nsIDBEvent()
{
  /* destructor code */
}

/* readonly attribute nsISupports source; */
NS_IMETHODIMP nsIDBEvent::GetSource(nsISupports **aSource)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIIDBEvent_h__ */
